import XCTest
@testable import SnapshotDemo
import SnapshotTesting

final class SnapshotDemoTests: XCTestCase {
    override class func setUp() {
//        SnapshotTesting.isRecording = true
    }
    
    func testExample() throws {
        let view = MyView()
        view.traitOverrides.preferredContentSizeCategory = .accessibilityExtraExtraExtraLarge
        
        assertSnapshot(
            matching: view,
            as: .image(
                targetSize: CGSize(
                    width: 300,
                    height: UIView.layoutFittingExpandedSize.height
                ),
                horizontalFittingPriority: .required,
                verticalFittingPriority: .fittingSizeLevel
            )
        )
    }
}

class MyView: UIView {
    override init(frame: CGRect) {
        super.init(frame: frame)
        setup()
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    lazy var label: UILabel = {
        let label = UILabel()
        label.text = "Example multiline text without any kind of joke in it"
        label.font = UIFont.preferredFont(forTextStyle: .body)
        label.numberOfLines = 0
        return label
    }()
    
    lazy var stackView: UIStackView = {
        let stackView = UIStackView()
        stackView.axis = .horizontal
        stackView.translatesAutoresizingMaskIntoConstraints = false
        return stackView
    }()
    
    func setup() {
        stackView.addArrangedSubview(label)
        addSubview(stackView)
        
        NSLayoutConstraint.activate([
            layoutMarginsGuide.leadingAnchor.constraint(equalTo: stackView.leadingAnchor),
            layoutMarginsGuide.trailingAnchor.constraint(equalTo: stackView.trailingAnchor),
            layoutMarginsGuide.topAnchor.constraint(equalTo: stackView.topAnchor),
            layoutMarginsGuide.bottomAnchor.constraint(equalTo: stackView.bottomAnchor)
        ])
    }
    
    override func systemLayoutSizeFitting(_ targetSize: CGSize, withHorizontalFittingPriority horizontalFittingPriority: UILayoutPriority, verticalFittingPriority: UILayoutPriority) -> CGSize {
        // you can set up a breakpoint here and get a proof that traitCollection is default, without updated preferredContentSizeCategory
        super.systemLayoutSizeFitting(
            targetSize, 
            withHorizontalFittingPriority: horizontalFittingPriority,
            verticalFittingPriority: verticalFittingPriority
        )
    }
}
